/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Yahoo! Mail Notifier code.
 *
 * The Initial Developer of the Original Code is
 * Michael Kraft.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function ym_login() {
  this.ym_prefs = new ym_prefs();
  this.newUserName = null;
  this.newPassword = null;
  this.currentDefaultUser = "";
  this.currentDefaultUserPw = "";
  this.attemptedLogin = false;

  // NsiObserver object
  this.observer = {
    observe : function(aSubject, aTopic, aData) {
      gYm_login.observe(aSubject, aTopic, aData);
    }
  }
}

/**
 * Retrieve the user preferences
 */
ym_login.prototype.doLoad = function() {
  var prefs = this.ym_prefs;
  var menulist = document.getElementById("ym-notifier-username");
  var userName = prefs.getPref(prefs.PREF_DEFAULT_USER);
  var users = prefs.getPref(prefs.PREF_USERS).split(" ");
  for (var i=0; i < users.length; i++) {
    menulist.appendItem(users[i]);
  }

  if (userName) {
    this.currentDefaultUser = userName;
    document.getElementById("ym-notifier-username").value = userName;
    var userInfo = prefs.loadUser(userName);
    if (userInfo) {
      this.currentDefaultUserPw = userInfo.password;
      document.getElementById("ym-notifier-password").value = userInfo.password;
      document.getElementById("ym-notifier-remember-pw").checked = userInfo.remember_pw;
      document.getElementById("ym-notifier-override").checked = userInfo.overrideURL;
      document.getElementById("ym-notifier-mailURL").value = userInfo.mailURL;
      delete userInfo;
    }
  }
  // Can't disable default, must create a new default and enable it there
  document.getElementById("ym-notifier-make-default").checked = true;
  document.getElementById("ym-notifier-make-default").setAttribute("disabled",true);

  if (!document.getElementById("ym-notifier-username").value || 
      !document.getElementById("ym-notifier-password").value) {
    document.getElementById("login").setAttribute("disabled",true);
    document.getElementById("ym-delete").setAttribute("disabled",true);
  }

  // Enable/disable mailurl field
  this.toggleMailURL();

  return true;
}

ym_login.prototype.doOK = function() {
  this.newUserName = document.getElementById("ym-notifier-username").value;
  this.newPassword = document.getElementById("ym-notifier-password").value;

  // username and or password changed
  if ((this.newUserName != this.currentDefaultUser) ||
      (this.newPassword != this.currentDefaultUserPw)) {

    this.attemptedLogin = true;
    
    // Call login to check login info, if login is successful we will get a login
    // notification, otherwise we will get a logout notification
    var observerService = Components.classes["@mozilla.org/observer-service;1"].
                          getService(Components.interfaces.nsIObserverService);
                          
    // logout first before we add observer because the login function might logout first
    window.opener.gYm_notifier.logout(this.newUserName);
                              
    observerService.addObserver(gYm_login.observer,"ym-gui-change",false);   

    window.opener.gYm_notifier.login(this.newUserName, this.newPassword, 
                                     document.getElementById("ym-notifier-remember-pw").checked);
    return false;
  }
  // no username/password change so just save updated data.
  else this.saveUser(this.newUserName);
}

/*
 * Observer functions
 */  
ym_login.prototype.observe = function(aSubject, aTopic, aData) {
	
  if (aTopic == "ym-gui-change") {
  	aData = aData.split(" ");
  	
    if (aData[0] == "login") {
      this.saveUser(aData[1]);
    }
    else if (aData[0] == "logout") {
      this.doCleanup();
    }
  }
}

/**
 * Canceled
 */
ym_login.prototype.doCancel = function() {
	var prefs = this.ym_prefs;
	
  this.doCleanup();
  
  // if user tried to change users and failed log back in with default user
  if (this.attemptedLogin) {
    var userName = prefs.getPref(prefs.PREF_DEFAULT_USER);
    if (userName) {
      var userInfo = prefs.loadUser(userName);
      if (userInfo && userInfo.Password)  {
        window.opener.gYm_notifier.login(userName,userInfo.Password,userInfo.remember_pw);
        delete userInfo;
      }
    }
  }

  return true;
}

/**
 * If user logged in successfully store info
 */
ym_login.prototype.saveUser = function(username) {
	
	// if this username is the one that logged in
	if (this.newUserName == username) {
    this.ym_prefs.saveUser(this.newUserName, this.newPassword, 
                   document.getElementById("ym-notifier-remember-pw").checked,
                   document.getElementById("ym-notifier-override").checked,
                   document.getElementById("ym-notifier-mailURL").value,
                   document.getElementById("ym-notifier-make-default").checked);

    this.doCleanup();
    setTimeout("window.close()", 1000)
  }
}

/**
 * Cleanup any observers
 */
ym_login.prototype.doCleanup = function() {
  // Remove observers
  try {
    var observerService = Components.classes["@mozilla.org/observer-service;1"].
                          getService(Components.interfaces.nsIObserverService);
    observerService.removeObserver(gYm_login.observer,"ym-gui-change");   
  }
  catch (ex) {}
}

/*
 * Delete the user info
 */
ym_login.prototype.deleteUser = function() {
  
  if (confirm(document.getElementById("ym-notifierStrings").getString("confirmdelete"))) {
 
    var prefs = this.ym_prefs;
    var menulist = document.getElementById("ym-notifier-username");

    // if deleting default user logout
    if (menulist.value == this.currentDefaultUser) window.opener.gYm_notifier.logout(this.currentDefaultUser);

    prefs.deleteUser(menulist.value);
    menulist.removeItemAt(menulist.selectedIndex);
    menulist.selectedIndex = 0;

    this.userNameChanged();
  }
}

/*
 * Retreive password and remember password settings on username change
 */
ym_login.prototype.userNameChanged = function() {
  var prefs = this.ym_prefs;
  var userName = document.getElementById("ym-notifier-username").value;
  var defaultUser = prefs.getPref(prefs.PREF_DEFAULT_USER);
  var foundInfo = false;

  try {
    if (userName) {
      var userInfo = prefs.loadUser(userName);
      if (userInfo) {
        document.getElementById("ym-notifier-password").value = userInfo.password;
        document.getElementById("ym-notifier-remember-pw").checked = userInfo.remember_pw;
        document.getElementById("ym-notifier-make-default").checked = (userName == defaultUser);
        document.getElementById("ym-notifier-make-default").setAttribute("disabled",(userName == defaultUser));
        document.getElementById("ym-delete").setAttribute("disabled",false);
        foundInfo = true;
        delete userInfo;
      }
    }
  }
  catch (ex) {}
  finally {
  	if (!foundInfo) {
      document.getElementById("ym-notifier-password").value = "";
      document.getElementById("ym-notifier-remember-pw").checked = false;
      document.getElementById("ym-notifier-make-default").checked = !defaultUser;
      document.getElementById("ym-notifier-make-default").setAttribute("disabled",!defaultUser);
      document.getElementById("ym-delete").setAttribute("disabled",true);
  	}

    this.updateLoginButton();
  }
}

/*
 * Enable/Disable login button 
 */
ym_login.prototype.updateLoginButton = function() {
  if (document.getElementById("ym-notifier-username").value && 
      document.getElementById("ym-notifier-password").value)
    document.getElementById("login").setAttribute("disabled",false);
  else
    document.getElementById("login").setAttribute("disabled",true);
}

/**
  * Disable/Enable override mail URL
  */
ym_login.prototype.toggleMailURL = function() {
	if (document.getElementById("ym-notifier-override").checked)
	  document.getElementById("ym-notifier-mailURL").removeAttribute("disabled");
	else
	  document.getElementById("ym-notifier-mailURL").setAttribute("disabled", true);
}


var gYm_login = new ym_login();
